package com.smg.persistwork.model.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Descripci dels camps que formen una taula.
 * @author Sergio Maeso
 * @version 1.0
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.FIELD,ElementType.ANNOTATION_TYPE})
public @interface Columna {

	/**
	 * Nom del camp, en cas que sigui diferent a l'atribut.
	 * @return nom del camp.
	 */
	String nom() default "";
	
	/**
	 * Tamany del camp.
	 * @return valor per defecte
	 */
	int tamany() default Integer.MAX_VALUE;
	
	/**
	 * Precisi del camp. Noms per als camps de tipus numric.
	 * @return precisi
	 */
	int precisio() default 0;
	
	/** Indicador de si es tracta d'un camp obligatori.
	 * @return obligatori
	 */
	boolean obligatori() default false;
	
	
	/**
	 * Valor per defecte que pot pendre el camp. Serveix per a camps text i numrics.
	 * @return valor per defecte
	 */
	String valorDefecte() default "";
	
	
}
