package com.smg.persistwork.model.mapeig;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Classe Equivalncia columna per al motor de base de dades MySql
 * 
 * @author Sergio Maeso
 * @version 1.0
 * 
 */
public class EquivalenciaColMySql extends EquivalenciaColumna {

	/**
	 * Format Numric sense precisi
	 */
	private final String INT = "INT";

	/**
	 * Format numric amb precisi
	 */
	private final String DECIMAL = "DECIMAL";
	/**
	 * Format de tipus data
	 */
	private final String DATETIME = "DATETIME";
	/**
	 * Format de tipus text.
	 */
	private final String VARCHAR = "VARCHAR";

	protected EquivalenciaColMySql() {
		super();

	}

	@Override
	protected String getColumnaBigDecimal() {

		return DECIMAL;
	}

	@Override
	protected String getColumnaData() {

		return DATETIME;
	}

	@Override
	protected String getColumnaText() {

		return VARCHAR;
	}

	@Override
	protected String getColumnaInteger() {

		return INT;
	}

	@Override
	protected Object getValor(Object valBD, String tipus)
			throws PersistWorkException {

		if (valBD != null && valBD instanceof BigDecimal) {
			if ("int".equals(tipus) || "java.lang.Integer".equals(tipus)) {
				return ((BigDecimal) valBD).intValue();
			}

			if ("java.lang.Long".equals(tipus) || "long".equals(tipus)) {
				return ((BigDecimal) valBD).longValue();
			}

			if ("double".equals(tipus) || "java.lang.Double".equals(tipus)) {

				return ((BigDecimal) valBD).doubleValue();
			}
			if ("java.lang.Float".equals(tipus) || "float".equals(tipus)) {
				return ((BigDecimal) valBD).floatValue();
			}
			if ("java.math.BigDecimal".equals(tipus)) {
				return valBD;
			}
		}

		if (valBD != null && valBD instanceof Long) {
			if ("int".equals(tipus) || "java.lang.Integer".equals(tipus)) {
				return ((Long) valBD).intValue();
			}

			if ("java.lang.Long".equals(tipus) || "long".equals(tipus)) {
				return valBD;
			}

			throw new PersistWorkException(PersistWorkException.MAP_1,
					PersistWorkException.MAP1_TXT);
		}

		if (valBD != null && valBD instanceof Integer) {
			if ("int".equals(tipus) || "java.lang.Integer".equals(tipus)) {
				return valBD;
			}

			if ("java.lang.Long".equals(tipus) || "long".equals(tipus)) {
				return new Long((Integer) valBD);
			}
			
		}

		if (valBD != null && valBD instanceof java.sql.Date
				|| valBD instanceof Timestamp) {
			if ("java.util.GregorianCalendar".equals(tipus)) {
				Calendar cal = GregorianCalendar.getInstance();
				cal.setTime((java.sql.Timestamp) valBD);
				return cal;
			}

			if ("java.util.Date".equals(tipus)) {
				return new Date(((java.sql.Timestamp) valBD).getTime());
			}
		}

		if (valBD != null && valBD instanceof String) {
			return valBD;
		}
		return null;
	}

}
