package com.smg.persistwork.model.mapeig;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Classe Equivalencia Columna per al motor de base de dades
 * Oracle
 * @author Sergio Maeso
 * @version 1.0
 */
public class EquivalenciaColOracle extends EquivalenciaColumna {

	
	/**
	 * Format de camp de tipus numric amb i sense precisi
	 */
	private final String NUMBER = "NUMBER";
	/**
	 * Format de camp de tipus data
	 */
	private final String DATE = "DATE";	
	/**
	 * Format de camp de tipus text
	 */
	private final String VARCHAR2 = "VARCHAR2";
	
	
	protected EquivalenciaColOracle() {
		super();
		
	}

	@Override
	protected String getColumnaBigDecimal() {
	
		return NUMBER;
	}

	@Override
	protected String getColumnaData() {
	
		return DATE;
	}

	@Override
	protected String getColumnaText() {
	
		return VARCHAR2;
	}

	@Override
	protected String getColumnaInteger() {
	
		return NUMBER;
	}
	
	
	/**
	 * @param valBD
	 * @param tipus
	 * @return
	 */
	@Override
	protected Object getValor(Object valBD, String tipus)  throws PersistWorkException{

		if (valBD != null && valBD instanceof BigDecimal) {
			if ("int".equals(tipus) || "java.lang.Integer".equals(tipus)) {
				return ((BigDecimal) valBD).intValue();
			}

			if ("java.lang.Long".equals(tipus) || "long".equals(tipus)) {
				return ((BigDecimal) valBD).longValue();
			}

			if ("double".equals(tipus) || "java.lang.Double".equals(tipus)) {

				return ((BigDecimal) valBD).doubleValue();
			}
			if ("java.lang.Float".equals(tipus) || "float".equals(tipus)) {
				return ((BigDecimal) valBD).floatValue();
			}
			if ("java.math.BigDecimal".equals(tipus)) {
				return valBD;
			}

		}
		
		if (valBD!=null && valBD instanceof java.sql.Date || valBD instanceof Timestamp){
			if ("java.util.GregorianCalendar".equals(tipus)){
				Calendar  cal = GregorianCalendar.getInstance();  
				cal.setTime((java.sql.Timestamp)valBD);
				return cal;				
			}
				
			if ("java.util.Date".equals(tipus)){
				return new Date(((java.sql.Timestamp)valBD).getTime());
			}

		}
		
		if (valBD!=null && valBD instanceof String){
			if ("java.lang.String".equals(tipus))
				return valBD;
		}
		return null;
	}

}
