package com.smg.persistwork.model.mapeig;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Utilitat de mapeig per a les consultes natives SQL.
 * @author Sergio Maeso
 * 
 * @param <T> objecte que ha de retornar el mapeig
 */
public class MapeigSql<T> {

	/**
	 * Equivalncia de camps de base de dades i de l'objecte java.
	 */
	private Map<String, String> camps;

	/**
	 * Objecte a on fem la equivalncia
	 */
	private T object;

	/**
	 * Equivalncia de columna segons el motor de base de dades
	 */
	private EquivalenciaColumna equiv;

	/**
	 * Constructor que prepara el mapeig.
	 * @param camps
	 * @param object
	 * @throws PersistWorkException 
	 */
	public MapeigSql(Map<String, String> camps, T object, String idEntorn) throws PersistWorkException {
		super();
		this.camps = camps;
		this.object = object;
		if (idEntorn == null)
			equiv = EquivalenciaColumna.getNewEquivalCol();
		else
			equiv = EquivalenciaColumna.getNewEquivalCol(idEntorn);
		camps = new HashMap<String, String>();
	}

	/**
	 * Constructor
	 * @param object
	 * @throws PersistWorkException 
	 */
	public MapeigSql(T object, String idEntorn) throws PersistWorkException {
		super();
		this.object = object;
		if (idEntorn == null)
			equiv = EquivalenciaColumna.getNewEquivalCol();
		else
			equiv = EquivalenciaColumna.getNewEquivalCol(idEntorn);
		camps = new HashMap<String, String>();
	}

	/**
	 * Setejador de la equivalncia dels camps de base de dades i de
	 * l'objecte.
	 * @param camps
	 */
	public void setCamps(Map<String, String> camps) {
		this.camps = camps;
	}

	/**
	 * Retorna una instncia de l'objecte a partir d'un ResultSet
	 * i segons el mapeig definit.
	 * @param rs resultat de la consulta
	 * @return instncia de l'objecte
	 * @throws PersistWorkException 
	 */
	public T getInstance(ResultSet rs) throws PersistWorkException {

		T ret = null ;
		try {
			ret = (T) object.getClass().newInstance();

			for (String nomBd : camps.keySet()) {

				Field f;
				Object valBD = rs.getObject(nomBd);
				String nomcamp = camps.get(nomBd);
				String nomMetode;
		

				if (valBD != null) {
					if (nomcamp.indexOf(".") == -1) {
						f = this.object.getClass().getDeclaredField(nomcamp);
						nomMetode = "set"
								+ f.getName().substring(0, 1).toUpperCase()
								+ f.getName().substring(1);
						Method mthd = ret.getClass().getDeclaredMethod(
								nomMetode, f.getType());

						mthd.invoke(ret, equiv.getValor(valBD, f.getType()
								.getCanonicalName()));

					} else {
						f = this.object.getClass().getDeclaredField(
								nomcamp.substring(0, nomcamp.indexOf(".")));
						nomMetode = "set"
								+ f.getName().substring(0, 1).toUpperCase()
								+ f.getName().substring(1);
						String nomMetodeGet = "get"
								+ f.getName().substring(0, 1).toUpperCase()
								+ f.getName().substring(1);
						Method mthd = ret.getClass().getDeclaredMethod(
								nomMetodeGet);
						Object objCF = mthd.invoke(ret);
						if (objCF == null) {
							objCF = f.getType().newInstance();
						}
						String altrecamp = nomcamp.substring(nomcamp
								.indexOf(".") + 1);
						Field f2 = objCF.getClass().getDeclaredField(altrecamp);
						String nomMetode2 = "set"
								+ f2.getName().substring(0, 1).toUpperCase()
								+ f2.getName().substring(1);
						Method mthd2 = objCF.getClass().getDeclaredMethod(
								nomMetode2, f2.getType());
						mthd2.invoke(objCF, equiv.getValor(valBD, f2.getType()
								.getCanonicalName()));
						mthd = ret.getClass().getDeclaredMethod(
								nomMetode,f.getType());
						mthd.invoke(ret, objCF);
					}

				}

			}

			return ret;

		} catch (InstantiationException e) {					
			throw new PersistWorkException(PersistWorkException.MAP_4,
					PersistWorkException.MAP4_TXT);
		} catch (IllegalAccessException e) {		
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (SecurityException e) {				
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (NoSuchFieldException e) {
			throw new PersistWorkException(PersistWorkException.MAP_2,
					PersistWorkException.MAP2_TXT);
		} catch (NoSuchMethodException e) {
			throw new PersistWorkException(PersistWorkException.MAP_6,
					PersistWorkException.MAP6_TXT);
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.MAP_8,
					PersistWorkException.MAP8_TXT);
		} catch (IllegalArgumentException e) {	
			throw new PersistWorkException(PersistWorkException.MAP_3,
					PersistWorkException.MAP3_TXT);
		} catch (InvocationTargetException e) {		
			throw new PersistWorkException(PersistWorkException.MAP_7,
					PersistWorkException.MAP7_TXT);
		}
		

	}

}
