package com.smg.persistwork.model.mapeig;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.model.annotations.ClauForanea;
import com.smg.persistwork.model.annotations.ClauPrimaria;
import com.smg.persistwork.model.annotations.Columna;
import com.smg.persistwork.model.annotations.Taula;

/**
 * Objecte Mapejat que es crea a partir de l'objecte que t les
 * anotacions de la base de dades. Aquest objecte genera totes les sentncies
 * necessries per a realitzar les accions amb la base de dades.
 * @author Sergio Maeso
 * @version 1.0
 */
public class ObjecteMapejat {

	/**
	 * Sentncia per a insertar un registre
	 */
	private String sqlInsert;

	/**
	 * Sentncia per a esborrar un registre
	 */
	private String sqlDelete;

	/**
	 * Sentncia per a modificar un registre
	 */
	private String sqlUpdate;

	/**
	 * Sentncia per a fer select d'un registre.
	 */
	private String sqlSelect;

	/**
	 * Sentncia per a crear la taula
	 */
	private String sqlCreate;

	/**
	 * Sentncia per a esborra la taula
	 */
	private String sqlDrop;

	/**
	 * Sentncia per a crear la clau primria
	 */
	private String sqlCreateCP;

	/**
	 * Retorna la sentncia a on estan noms els camps.
	 */
	private String sqlCamps;

	/**
	 * Nom de la taula
	 */
	private String nomTaula;

	/**
	 * Posici dintre del registre
	 */
	private int posicio;

	/**
	 * Equivalncies entre els camps de la taula i els camps de l'objecte
	 */
	private Map<String, Field> equivalencies = new HashMap<String, Field>();

	/**
	 * Classe de l'objecte que mapeja
	 */
	private Class<? extends Object> objecte;

	/**
	 * Equivalncia entre els camps de la clau primria i els camps de l'objecte.
	 */
	private Map<String, Field> equivClausPrimaries = new HashMap<String, Field>();

	/**
	 * Equivalncia entre els camps de la clau foranea i els camps de l'objecte.
	 */
	private Map<String, Field> equivForanees = new HashMap<String, Field>();

	/**
	 * Objectes que mapejen les claus foranees que t aquest objecte
	 */
	private Map<String, ObjecteMapejat> objsForanees = new HashMap<String, ObjecteMapejat>();

	/**
	 * Equivalncia de columna
	 */
	private EquivalenciaColumna equiv;
	
	/**
	 * Indica el nom del camp que s'incrementa de forma automtica sense seqncial
	 */
	private String incrementLastId;

	/**
	 * Equivalncia entre els camps.
	 */
	private Map<String, String> equivcamps = new HashMap<String, String>();

	/**
	 * Constructor
	 * @param objecte
	 * @param id
	 * @throws PersistWorkException
	 */
	public <T> ObjecteMapejat(T objecte, String... id)
			throws PersistWorkException {

		this.posicio = 0;
		inicialitza(objecte, id);

	}

	/**
	 * Inicialitzaci de l'objecte mapejat.
	 * @param objecte
	 * @param id
	 * @throws PersistWorkException
	 */
	private <T> void inicialitza(T objecte, String... id)
			throws PersistWorkException {
		this.objecte = objecte.getClass();

		if (id.length == 0)
			equiv = EquivalenciaColumna.getNewEquivalCol();
		else
			equiv = EquivalenciaColumna.getNewEquivalCol(id[0]);
		
		if (!objecte.getClass().isAnnotationPresent(Taula.class)) {
			throw new PersistWorkException(PersistWorkException.MAP_9, PersistWorkException.MAP9_TXT);
		}

		Taula t = this.objecte.getAnnotation(Taula.class);

		if (!"".equals(t.nom())) {
			this.nomTaula = t.nom().toUpperCase();
		} else {
			this.nomTaula = objecte.getClass().getSimpleName().toUpperCase();
		}

		for (Field f : objecte.getClass().getDeclaredFields()) {

			if (f.isAnnotationPresent(Columna.class)) {

				Columna c = f.getAnnotation(Columna.class);
				String nom = !"".equals(c.nom()) ? c.nom().toUpperCase() : f
						.getName().toUpperCase();
				equivalencies.put(nom, f);
				equivcamps.put(f.getName().toUpperCase(), nom);
			}

			if (f.isAnnotationPresent(ClauPrimaria.class)) {
				Columna c = f.getAnnotation(Columna.class);
				
				String nom = !"".equals(c.nom()) ? c.nom().toUpperCase() : f
						.getName().toUpperCase();
				equivClausPrimaries.put(nom, f);
				
			}

			if (f.isAnnotationPresent(ClauForanea.class)) {
		
				ClauForanea cf = f.getAnnotation(ClauForanea.class);
				String nom = !"".equals(cf.nomCamp()) ? cf.nomCamp()
						.toUpperCase() : f.getName().toUpperCase();
				equivalencies.put(nom, f);
				equivForanees.put(nom, f);
				equivcamps.put(f.getName().toUpperCase(), nom);
				Constructor[] cts = f.getType().getConstructors();

				for (Constructor ct : cts) {

					try {
						RegistreObjectes regs;
						if (id.length != 0)
							regs = RegistreObjectesFactory.getRegistre(id[0]);
						else
							regs = RegistreObjectesFactory.getRegistre(null);
						if (ct.getGenericParameterTypes().length == 0) {
							Object obj = ct.newInstance();
							ObjecteMapejat objmap = regs.getObjecte(obj);
							objsForanees.put(nom, objmap);
						}

					} catch (IllegalArgumentException e) {
						throw new PersistWorkException(PersistWorkException.MAP_3,
								PersistWorkException.MAP3_TXT);
					} catch (InstantiationException e) {
						throw new PersistWorkException(PersistWorkException.MAP_4,
								PersistWorkException.MAP4_TXT);
					} catch (IllegalAccessException e) {
						throw new PersistWorkException(PersistWorkException.MAP_5,
								PersistWorkException.MAP5_TXT);
					} catch (InvocationTargetException e) {
						throw new PersistWorkException(PersistWorkException.MAP_7,
								PersistWorkException.MAP7_TXT);
					}
				}

			}
		}
	}

	/**
	 * Retorna un camp de l'objecte a partir d'un camp de la bd.
	 * @param nomCamp
	 * @return camp de l'objecte
	 */
	public Field getCamp(String nomCamp) {

		return equivalencies.get(nomCamp);
	}

	/**
	 * Retorna la sentncia d'insertar un registre. Noms
	 * es genera un cop.
	 * @return sentncia insert
	 */
	public String getSqlInsert() {

		if (sqlInsert == null) {
			StringBuilder sbInsert = new StringBuilder();
			sbInsert.append("INSERT INTO ").append(nomTaula).append(" (");
			boolean ficarComa = false;
			ArrayList<String> campsInsert = new ArrayList<String>();
			for (String nom : equivalencies.keySet()) {
				Field f = equivalencies.get(nom);

				boolean inserta = true;
			

				if (f.isAnnotationPresent(ClauPrimaria.class)) {
					ClauPrimaria cp = f.getAnnotation(ClauPrimaria.class);
					switch (cp.tipusClau()) {

					case VALOR_MAX:
						campsInsert.add("@$SELECT MAX(" + nom + ")+1 FROM "
								+ nomTaula + "$#" + nom + "#@");
						break;
					case VALOR_ASSIGNAT:
						campsInsert.add("@" + nom + "@");
						break;
					case AUTO_INCREMENT:
						if (equiv instanceof EquivalenciaColOracle){
							campsInsert.add("@$SELECT " + cp.nomSequencia()
									+ ".NEXTVAL FROM DUAL$#" + nom + "#@");	
						} else{
							inserta = false;
							this.incrementLastId = nom;
						}
						
						break;
					}

				} else if (f.isAnnotationPresent(ClauForanea.class)) {
					ClauForanea cf = f.getAnnotation(ClauForanea.class);
					nom = !"".equals(cf.nomCamp()) ? cf.nomCamp().toUpperCase()
							: f.getName().toUpperCase();
					campsInsert.add("@" + nom + "@");
				} else {
					campsInsert.add("@" + nom + "@");
				}

				if (inserta){
					if (ficarComa) {
						sbInsert.append(",");
					} else {
						ficarComa = true;
					}
					
					sbInsert.append(nom);
				}
			

			}

			sbInsert.append(") VALUES (");
			for (int i = 0; i < campsInsert.size(); i++) {
				if (i != 0)
					sbInsert.append(",");
				sbInsert.append(campsInsert.get(i));
			}
			sbInsert.append(")");
			sqlInsert = sbInsert.toString();
		}

		return sqlInsert;
	}

	/**
	 * Retorna la sentncia per a esborrar un registre.
	 * Noms es genera un cop.
	 * @return sentncia delete
	 */
	public String getSqlDelete() {

		if (sqlDelete == null) {
			StringBuilder sbDelete = new StringBuilder();
			sbDelete.append("DELETE FROM ").append(nomTaula).append(" WHERE ");
			boolean ficarComa = false;
			for (String nom : equivClausPrimaries.keySet()) {

				if (ficarComa) {
					sbDelete.append(",");
				} else {
					ficarComa = true;
				}

				sbDelete.append(nom).append(" = @").append(nom).append("@");
			}
			this.sqlDelete = sbDelete.toString();
		}

		return sqlDelete;
	}

	/**
	 * Retorna la sentncia per a fer una modificaci d'un registre.
	 * Noms es genera un cop.
	 * @return sentncia update
	 */
	public String getSqlUpdate() {

		if (sqlUpdate == null) {
			StringBuilder sb = new StringBuilder();

			sb.append("UPDATE ").append(nomTaula).append(" SET ");
			boolean ficarComa = false;
			for (String nom : equivalencies.keySet()) {
				Field f = equivalencies.get(nom);

				if (ficarComa) {
					sb.append(",");
				} else {
					ficarComa = true;
				}

				if (f.isAnnotationPresent(Columna.class)) {
					Columna c = f.getAnnotation(Columna.class);
					nom = !"".equals(c.nom()) ? c.nom().toUpperCase() : f
							.getName().toUpperCase();
				}
				sb.append(nom).append(" = ").append("@").append(nom)
						.append("@");

			}

			sb.append(" WHERE ");
			boolean ficarAnd = false;
			for (String nom : equivClausPrimaries.keySet()) {

				if (ficarAnd) {
					sb.append(" AND ");
				} else {
					ficarAnd = true;
				}

				sb.append(nom).append(" = ").append("@").append(nom)
						.append("@");

			}

			sqlUpdate = sb.toString();

		}

		return sqlUpdate;
	}

	/**
	 * Retorna noms la part del select a on s'indiquen els
	 * camps que es volen cercar.
	 * @return sentncia select dels camps.
	 */
	public String getSqlCamps() {

		if (sqlCamps == null) {
			StringBuilder sb = new StringBuilder();
			StringBuilder sbFrom = new StringBuilder();
			boolean ficarComa = false;

			sb.append("SELECT ");

			for (String nom : equivalencies.keySet()) {

				if (ficarComa) {
					sb.append(",");
				} else {
					ficarComa = true;
				}
				sb.append(getPrefix()).append(".").append(nom).append(" AS ")
						.append(getPrefix()).append(nom);

			}

			// CLAUS FORANEAS
			// LES FICAREM SI SON OBLIGATORIES AMB INNER JOIN
			// O LEFT JOIN SI SON OPTATIVES
			sbFrom.append(" FROM ").append(nomTaula).append(" ")
					.append(getPrefix());

			for (String stfor : equivForanees.keySet()) {
				Field fcf = equivForanees.get(stfor);
				ClauForanea cf = fcf.getAnnotation(ClauForanea.class);

				if (cf.obligatori()) {
					sbFrom.append(" INNER JOIN ");
				} else {
					sbFrom.append(" LEFT JOIN ");
				}
				ObjecteMapejat objm = objsForanees.get(stfor);
				sbFrom.append(objm.nomTaula).append(" ")
						.append(objm.getPrefix());
				sbFrom.append(" ON ").append(getPrefix()).append(".")
						.append(stfor).append("=");
				for (String st : objm.equivClausPrimaries.keySet()) {
					sbFrom.append(objm.getPrefix()).append(".").append(st);
				}
				for (String st : objm.equivalencies.keySet()) {
					if (ficarComa) {
						sb.append(",");
					} else {
						ficarComa = true;
					}
					sb.append(objm.getPrefix()).append(".").append(st)
							.append(" AS ").append(objm.getPrefix()).append(st);
				}
			}
			sb.append(sbFrom);
			sqlCamps = sb.toString();

		}

		return sqlCamps;
	}

	/**
	 * Retorna el select d'una clau primria
	 * @return sentncia select
	 * @throws PersistWorkException
	 */
	public String getSqlSelect() throws PersistWorkException {

		if (sqlSelect == null) {
			StringBuilder sb = new StringBuilder();
			sb.append(getSqlCamps());
			sb.append(" WHERE ");
			boolean ficarAnd = false;

			for (String nom : equivClausPrimaries.keySet()) {

				if (ficarAnd) {
					sb.append(" AND ");
				} else {
					ficarAnd = true;
				}

				sb.append(getPrefix()).append(".").append(nom).append(" = ")
						.append("@").append(nom).append("@");

			}
			sqlSelect = sb.toString();
		}

		return sqlSelect;
	}

	/**
	 * Retorna la part del select amb la condici del where
	 * @param <T>
	 * @param obj
	 * @return sentncia condici
	 * @throws PersistWorkException
	 */
	public <T> String getSqlCondCerca(T obj) throws PersistWorkException {

		StringBuilder sb = new StringBuilder();
		sb.append(getSqlCamps());
		sb.append(" WHERE ");
		boolean ficarAnd = false;

		try {
			for (Field camp : obj.getClass().getDeclaredFields()) {

				String nom = "";
				if (camp.isAnnotationPresent(ClauForanea.class)) {

					ClauForanea cf = camp.getAnnotation(ClauForanea.class);
					nom = !"".equals(cf.nomCamp()) ? cf.nomCamp().toUpperCase()
							: camp.getName().toUpperCase();

					String nomMetode = "get"
							+ camp.getName().substring(0, 1).toUpperCase()
							+ camp.getName().substring(1);
					String nomMetodeCF = null;
					for (Field f2 : camp.getType().getDeclaredFields()) {

						if (f2.isAnnotationPresent(ClauPrimaria.class)
								&& f2.isAnnotationPresent(Columna.class)) {
							nomMetodeCF = "get"
									+ f2.getName().substring(0, 1)
											.toUpperCase()
									+ f2.getName().substring(1);
						}
					}

					Method mthd = obj.getClass().getDeclaredMethod(nomMetode);
					Object objCF = mthd.invoke(obj);
					if (objCF!=null){
						Method mthdCF = objCF.getClass().getDeclaredMethod(
								nomMetodeCF);

						if (mthdCF.invoke(objCF) != null
								&& !equiv.isNullBasicType(camp.getType()
										.getCanonicalName(), mthdCF.invoke(objCF))) {

							if (ficarAnd) {
								sb.append(" AND ");
							} else {
								ficarAnd = true;
							}

							//ObjecteMapejat objm = objsForanees.get(nom);
							sb.append(getPrefix()).append(".").append(nom);
							sb.append(" = ").append("@").append(nom).append("@");

						}
					}
					

				} else if (camp.isAnnotationPresent(Columna.class)) {

					Columna c = camp.getAnnotation(Columna.class);
					nom = !"".equals(c.nom()) ? c.nom().toUpperCase() : camp
							.getName().toUpperCase();
					String nomMetode = "get"
							+ camp.getName().substring(0, 1).toUpperCase()
							+ camp.getName().substring(1);
					Method mthd = obj.getClass().getDeclaredMethod(nomMetode);

					if (mthd.invoke(obj) != null
							&& !equiv.isNullBasicType(camp.getType()
									.getCanonicalName(), mthd.invoke(obj))) {

						if (ficarAnd) {
							sb.append(" AND ");
						} else {
							ficarAnd = true;
						}

						sb.append(getPrefix()).append(".").append(nom)
								.append(" = ").append("@").append(nom)
								.append("@");

					}

				}

			}

		} catch (SecurityException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (NoSuchMethodException e) {
			throw new PersistWorkException(PersistWorkException.MAP_6,
					PersistWorkException.MAP6_TXT);
		} catch (IllegalArgumentException e) {
			throw new PersistWorkException(PersistWorkException.MAP_3,
					PersistWorkException.MAP3_TXT);
		} catch (IllegalAccessException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (InvocationTargetException e) {
			throw new PersistWorkException(PersistWorkException.MAP_7,
					PersistWorkException.MAP7_TXT);
		}

		return sb.toString();

	}

	/**
	 * Retorna la sentncia per a crear una taula.
	 * @return sentncia creaci taula
	 * @throws PersistWorkException
	 */
	public String getSqlCreate() throws PersistWorkException {

		if (sqlCreate == null) {

			StringBuilder sbCreate = new StringBuilder();
			sbCreate.append("CREATE TABLE ").append(nomTaula).append(" (");
			boolean ficarComa = false;

			for (String nom : equivalencies.keySet()) {

				Field f = equivalencies.get(nom);
				Columna c = null;
				ClauForanea cf = null;
				if (ficarComa) {
					sbCreate.append(", ");
				} else {
					ficarComa = true;
				}

				sbCreate.append(nom).append(" ");
				if (equivForanees.containsKey(nom)) {
					Field f2 = equivForanees.get(nom);
					cf = f2.getAnnotation(ClauForanea.class);
					ObjecteMapejat obj2 = objsForanees.get(nom);

					for (String nom2 : obj2.equivClausPrimaries.keySet()) {
						Field fcp = obj2.equivClausPrimaries.get(nom2);
						c = fcp.getAnnotation(Columna.class);
						sbCreate.append(equiv.getTipusColumna(fcp.getType()
								.getCanonicalName()));
					}

				} else {
					c = f.getAnnotation(Columna.class);
					sbCreate.append(equiv.getTipusColumna(f.getType()
							.getCanonicalName()));
				}

				if (c.tamany() > 0 && c.tamany() < Integer.MAX_VALUE) {
					if (c.precisio() > 0) {
						if (equiv.isCampNumericSensePrecisio(f.getType()
								.getCanonicalName())) {
							throw new PersistWorkException(PersistWorkException.MAP_10,
									PersistWorkException.MAP10_TXT);

						}
						sbCreate.append("(").append(c.tamany()).append(",")
								.append(c.precisio()).append(")");
					} else {
						sbCreate.append("(").append(c.tamany()).append(")");
					}
				}

				if (!"".equals(c.valorDefecte())) {
					if (equiv instanceof EquivalenciaColOracle
							|| (equiv instanceof EquivalenciaColMySql && !equiv
									.isCampDate(f.getType().getCanonicalName()))) {
						if (equiv.isCampText(f.getType().getCanonicalName())) {
							sbCreate.append(" DEFAULT '")
									.append(c.valorDefecte()).append("'");
						} else {
							sbCreate.append(" DEFAULT ").append(
									c.valorDefecte());
						}

					}

				}

				if ((c.obligatori() && cf == null)
						|| (cf != null && cf.obligatori())) {
					sbCreate.append(" NOT NULL");
				}

			}

			sbCreate.append(")");

			this.sqlCreate = sbCreate.toString();
		}

		return sqlCreate;
	}

	/**
	 * Retorna la sentncia per a esborrar una taula
	 * @return sentncia esborrar taula
	 */
	public String getSqlDrop() {
		if (sqlDrop == null) {
			StringBuilder sbDrop = new StringBuilder();
			sbDrop.append("DROP TABLE ").append(this.nomTaula);
			sqlDrop = sbDrop.toString();
		}
		return sqlDrop;
	}

	/**
	 * Retorna la sentncia per a crear la clau primria
	 * @return sentncia per a crear la clau primria
	 * @throws PersistWorkException
	 */
	public String getSqlCreateCP() throws PersistWorkException {

		if (sqlCreateCP == null) {
			StringBuilder sbCreateCP = new StringBuilder();

			if (equivClausPrimaries.size() > 0) {

				sbCreateCP.append("ALTER TABLE " + nomTaula
						+ " ADD PRIMARY KEY(");
				boolean ficarComa = false;
				for (String nom : equivClausPrimaries.keySet()) {
					if (ficarComa)
						sbCreateCP.append(",");
					else
						ficarComa = true;
					sbCreateCP.append(nom);
				}
				sbCreateCP.append(")");
			}

			this.sqlCreateCP = sbCreateCP.toString();
		}

		return sqlCreateCP;
	}

	/**
	 * Sentncia per a crear el seqncial o el autoincrement.
	 * @return sentncies per als sequncials o autoincrements
	 * @throws PersistWorkException
	 */
	public List<String> getCreateSeqCP() throws PersistWorkException {

		List<String> st = new ArrayList<String>();

		if (equivClausPrimaries.size() > 0) {

			for (String nom : equivClausPrimaries.keySet()) {
				StringBuilder sbSequencia = new StringBuilder();
				Field f = equivClausPrimaries.get(nom);
				ClauPrimaria cp = f.getAnnotation(ClauPrimaria.class);

				if (cp != null && !"".equals(cp.nomSequencia())) {
					if (equiv instanceof EquivalenciaColOracle) {
						sbSequencia.append("CREATE SEQUENCE ").append(
								cp.nomSequencia());
						sbSequencia
								.append(" MINVALUE 1 START WITH 1 INCREMENT BY 1 NOCACHE");
						st.add(sbSequencia.toString());
					} else {
						Columna c = f.getAnnotation(Columna.class);
						sbSequencia.append("ALTER TABLE ").append(nomTaula)
								.append(" CHANGE COLUMN ").append(nom);
						sbSequencia
								.append(" ")
								.append(nom)
								.append(" ")
								.append(equiv.getTipusColumna(f.getType()
										.getCanonicalName()));
						if (c.tamany() > 0 && c.tamany() < Integer.MAX_VALUE) {
							if (c.precisio() > 0) {
								if (equiv.isCampNumericSensePrecisio(f
										.getType().getCanonicalName())) {
									throw new PersistWorkException(5,
											"tipo no soporta precisi");

								}
								sbSequencia.append("(").append(c.tamany())
										.append(",").append(c.precisio())
										.append(")");
							} else {
								sbSequencia.append("(").append(c.tamany())
										.append(")");
							}
						}

						if (!"".equals(c.valorDefecte())) {

							if (equiv
									.isCampText(f.getType().getCanonicalName())) {
								sbSequencia.append(" DEFAULT '")
										.append(c.valorDefecte()).append("'");
							} else {
								sbSequencia.append(" DEFAULT ").append(
										c.valorDefecte());
							}

						}

						if (c.obligatori()) {
							sbSequencia.append(" NOT NULL");
						}
						sbSequencia.append(" AUTO_INCREMENT ");
						st.add(sbSequencia.toString());
					}

				}

			}

		}

		return st;

	}

	/**
	 * Retorna la sentncia per a esborrar els seqncials,
	 * noms per a oracle.
	 * @return llista de sentncies.
	 * @throws PersistWorkException
	 */
	public List<String> getDropSeqCP() throws PersistWorkException {

		List<String> st = new ArrayList<String>();

		if (equiv instanceof EquivalenciaColOracle){
			if (equivClausPrimaries.size() > 0) {

				for (String nom : equivClausPrimaries.keySet()) {
					StringBuilder sbSequencia = new StringBuilder();
					Field f = equivClausPrimaries.get(nom);
					ClauPrimaria cp = f.getAnnotation(ClauPrimaria.class);
					if (cp != null && !"".equals(cp.nomSequencia())) {
						sbSequencia.append("DROP SEQUENCE ").append(
								cp.nomSequencia());
						st.add(sbSequencia.toString());
					}

				}

			}
	
		}
		return st;

	}

	/**
	 * Retorna les sentncies per a crear les claus foranees 
	 * @return sentncia claus foranees
	 * @throws PersistWorkException
	 */
	public List<String> getSqlCreateClausForaneas() throws PersistWorkException {

		List<String> llista = new ArrayList<String>();

		for (String nomCamp : equivForanees.keySet()) {
			StringBuilder sbCreateForanea = new StringBuilder();
			sbCreateForanea.append("ALTER TABLE ").append(this.nomTaula)
					.append(" ADD CONSTRAINT ");
			Field f = equivForanees.get(nomCamp);
			ClauForanea cf = f.getAnnotation(ClauForanea.class);
			ObjecteMapejat objmap = objsForanees.get(nomCamp);

			if (!"".equals(cf.nomClauForana())) {
				sbCreateForanea.append(cf.nomClauForana());
			} else {

				sbCreateForanea.append("FK_").append(this.nomTaula).append("_")
						.append(nomCamp);
			}
			sbCreateForanea.append(" FOREIGN KEY(").append(nomCamp)
					.append(") REFERENCES ");

			sbCreateForanea.append(objmap.nomTaula);

			for (String cp : objmap.equivClausPrimaries.keySet()) {
				sbCreateForanea.append("(").append(cp).append(")");
			}

			llista.add(sbCreateForanea.toString());

		}

		return llista;
	}

	/**
	 * Retorna una instncia de la classe a partir d'un resultset
	 * @param <T>
	 * @param rs
	 * @return instncia de la classe
	 */

	public <T> T getInstance(ResultSet rs) throws PersistWorkException {

		T objret = null;
		try {
			objret = (T) this.objecte.newInstance();

			for (String s : equivalencies.keySet()) {

				Field f = getCamp(s);
				try {
					Object valBD = rs.getObject(getPrefix() + s);
					if (valBD != null) {
						String nomMetode = "set"
								+ f.getName().substring(0, 1).toUpperCase()
								+ f.getName().substring(1);

						Method mthd = objret.getClass().getDeclaredMethod(
								nomMetode, f.getType());

						mthd.invoke(objret, equiv.getValor(valBD, f.getType()
								.getCanonicalName()));
					}
				} catch (SQLException sql){
					
					return objret;
				}
				

			}

			for (String s : equivForanees.keySet()) {

				ObjecteMapejat objfor = objsForanees.get(s);
				Object instfor = objfor.getInstance(rs);
				if (instfor != null) {
					Field f = getCamp(s);
					String nomMetode = "set"
							+ f.getName().substring(0, 1).toUpperCase()
							+ f.getName().substring(1);

					Method mthd = objret.getClass().getDeclaredMethod(
							nomMetode, f.getType());

					mthd.invoke(objret, instfor);
				}
			}

		} catch (InstantiationException e1) {
			throw new PersistWorkException(PersistWorkException.MAP_4,
					PersistWorkException.MAP4_TXT);
		} catch (IllegalAccessException e1) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (SecurityException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (NoSuchMethodException e) {
			throw new PersistWorkException(PersistWorkException.MAP_6,
					PersistWorkException.MAP6_TXT);
		} catch (IllegalArgumentException e) {
			throw new PersistWorkException(PersistWorkException.MAP_3,
					PersistWorkException.MAP3_TXT);
		} catch (InvocationTargetException e) {
			throw new PersistWorkException(PersistWorkException.MAP_7,
					PersistWorkException.MAP7_TXT);
		}

		return objret;
	}


	/**
	 * Informa d'un valor a l'objece indicant el valor i el camp.
	 * @param obj
	 * @param valor
	 * @param nomCamp
	 * @throws PersistWorkException
	 */
	public void informaValor(Object obj, Object valor, String nomCamp)
			throws PersistWorkException {

		Field camp = getCamp(nomCamp);
		String nomMetode = "set" + camp.getName().substring(0, 1).toUpperCase()
				+ camp.getName().substring(1);
		try {

			Method mthd = obj.getClass().getDeclaredMethod(nomMetode,
					camp.getType());
			mthd.invoke(obj,
					equiv.getValor(valor, camp.getType().getCanonicalName()));

		} catch (SecurityException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (NoSuchMethodException e) {
			throw new PersistWorkException(PersistWorkException.MAP_6,
					PersistWorkException.MAP6_TXT);
		} catch (IllegalArgumentException e) {
			throw new PersistWorkException(PersistWorkException.MAP_3,
					PersistWorkException.MAP3_TXT);
		} catch (IllegalAccessException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (InvocationTargetException e) {
			throw new PersistWorkException(PersistWorkException.MAP_7,
					PersistWorkException.MAP7_TXT);
		}

	}

	/**
	 * Retorna el nom del camp de BD a partir del nom del camp.
	 * @param stTractar
	 * @return nom del camp
	 */
	public String getNomCamp(String stTractar) {

		return equivcamps.get(stTractar);
	}

	/**
	 * Retorna la sentncia condicional traduda per al fwkql.
	 * @param sentwhere
	 * @return sentncia definitiva.
	 */
	public String getsqlCondTrad(String sentwhere, String prefixe) {

		String ret = new String(sentwhere);
		for (String equiv : equivcamps.keySet()) {

			if (equivForanees.containsKey(equivcamps.get(equiv))) {
				if (ret.indexOf(":" + prefixe + equiv + ".") != -1) {
					ObjecteMapejat objcf = objsForanees.get(equivcamps
							.get(equiv));
					ret = objcf.getsqlCondTrad(ret, equiv + ".");
				}
			} else {
				ret = ret.replace(":" + prefixe + equiv, getPrefix() + "."
						+ equivcamps.get(equiv));
			}
		}

		return ret;
	}

	/**
	 * Retorna el prefix de la taula de base de dades per a fer les
	 * consultes.
	 * @return
	 */
	private String getPrefix() {

		if (nomTaula.length() > 4)
			return nomTaula.substring(0, 4).toUpperCase() + posicio;
		else
			return nomTaula.substring(0) + posicio;
	}

	/**
	 * Retorna la posici de l'objecte. Aquest s nic i serveix per al
	 * prefix de la taula.
	 * @param posicio
	 */
	protected void setPosicio(int posicio) {
		this.posicio = posicio;
	}
	
	/**
	 * Retorna si s'incremente de forma automtica el id.
	 * Noms per a MySql
	 * @return s'incrementa automticament
	 */
	public boolean getIncrementLastId(){
		
		return incrementLastId!=null;
	}
	

	/**
	 * Informa del valor del lastId.
	 * Noms per MySql
	 * @param rs
	 * @param obj
	 * @throws PersistWorkException
	 */
	public <T> void setLastId(ResultSet rs, T obj) throws PersistWorkException{
		
		Field camp = equivalencies.get(incrementLastId);
		String nomMetode = "set" + camp.getName().substring(0, 1).toUpperCase()
				+ camp.getName().substring(1);
		
		try {

			Method mthd = obj.getClass().getDeclaredMethod(nomMetode,
					camp.getType());
			mthd.invoke(obj,
					equiv.getValor(rs.getObject(1), camp.getType().getCanonicalName()));

		} catch (SecurityException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (NoSuchMethodException e) {
			throw new PersistWorkException(PersistWorkException.MAP_2,
					PersistWorkException.MAP2_TXT);
		} catch (IllegalArgumentException e) {
			throw new PersistWorkException(PersistWorkException.MAP_3,
					PersistWorkException.MAP3_TXT);
		} catch (IllegalAccessException e) {
			throw new PersistWorkException(PersistWorkException.MAP_5,
					PersistWorkException.MAP5_TXT);
		} catch (InvocationTargetException e) {
			throw new PersistWorkException(PersistWorkException.MAP_7,
					PersistWorkException.MAP7_TXT);
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.MAP_8,
					PersistWorkException.MAP8_TXT);
		}
		
	}

}
