package com.smg.persistwork.model.mapeig;

import java.util.HashMap;
import java.util.Map;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Classe que registra tots els objectes que s'han mapejat per a poder-los
 * utilitzar quan es vulgui
 * @author Sergio Maeso
 * @version 1.0
 * 
 */
public class RegistreObjectes {

	/**
	 * Map on tenim tots els objectes mapejats
	 */
	private Map<String, ObjecteMapejat> objMapejats;
	
	/**
	 * Identificador en cas que hagi diversos registres
	 */
	private String id;

	/**
	 * 
	 */
	protected RegistreObjectes(String id) {
		this.id = id;
		objMapejats = new HashMap<String, ObjecteMapejat>();
	}

	/**
	 * Retorna el objecte a partir de la seva classe.
	 * @param obj
	 * @return objecte mapejat
	 * @throws PersistWorkException
	 */
	public final ObjecteMapejat getObjecte(Object obj)
			throws PersistWorkException {

		if (!existObjecte(obj)) {

			afegeixObjecte(obj);
		}
		return objMapejats.get(obj.getClass().toString());

	}

	/**
	 * Retorna si existeix l'objecte al registre
	 * @param <T>
	 * @param obj
	 * @return
	 */
	private <T> boolean existObjecte(T obj) {
		
		return objMapejats.containsKey(obj.getClass().toString());
	}

	/**
	 * Afegeix l'objecte al registre.
	 * @param <T>
	 * @param obj
	 * @throws PersistWorkException
	 */
	private <T> void afegeixObjecte(T obj) throws PersistWorkException {
		objMapejats.put(obj.getClass().toString(), creaObjecte(obj));
	}

	/**
	 * Crea l'objecte mapejat indicant-li la ppsici.
	 * @param <T>
	 * @param obj
	 * @return
	 * @throws PersistWorkException
	 */
	private <T> ObjecteMapejat creaObjecte(T obj) throws PersistWorkException {
		ObjecteMapejat objmap;
		if (id!=null)
			objmap = new ObjecteMapejat(obj, id);
		else
			objmap = new ObjecteMapejat(obj);
		objmap.setPosicio(objMapejats.size()+1);
		return objmap;
	}
}
