package com.smg.persistwork.nucli;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Realitza la interpretaci entre el llenguatge Fwkql i sql
 * El format de les sentncies s el segent:
 * SELECT NOMTAULA ON :CAMP=? I :CAMP!=? O :CAMP.CAMP=?
 * 
 * @author Sergio Maeso
 * @version 1.0
 * 
 */
public class FwkInterpreter {

	/**
	 * Sentncia que s'ha d'interpretar a Sql
	 */
	private String sentencia;

	/**
	 * Objecte de la sentncia.
	 */
	private Object obj;

	/**
	 * @param sentencia
	 */
	protected FwkInterpreter(String sentencia) {
		super();
		this.sentencia = sentencia;
	}

	/**
	 * Fa la traducci entre fwkql i sql
	 * @return
	 */
	protected String tradueix() {

		StringBuilder sb = new StringBuilder();
		
		int ultimPos = sentencia.replaceAll(" on "," ON ").indexOf(" ON ");
	
		while (ultimPos<sentencia.length()){
			int posI = sentencia.indexOf("'", ultimPos);
			if (posI==-1){
				sb.append(sentencia.substring(ultimPos).toUpperCase());
				ultimPos=sentencia.length();
			} else {
				sb.append(sentencia.substring(ultimPos, posI).toUpperCase());
				ultimPos = posI + 1;
				int posF = sentencia.indexOf("'", ultimPos);
				sb.append(sentencia.substring(posI, posF+1));
				ultimPos = posF + 1;
			}
		}
										
		return sb.toString().replaceAll(" ON ", " WHERE ")
				.replaceAll(" I ", " AND ")
				.replaceAll(" O ", " OR ")
				.replaceAll(" SEMBLANT ", " LIKE ")
				.replaceAll(" NO SEMBLANT ", " NOT LIKE ")
				.replaceAll(" ES NUL ", " IS NULL ")
				.replaceAll(" NO ES NULL ", " IS NOT NULL ");
	
	}

	/**
	 * Retorna l'objecte sobre el qual es fa la sentncia fwkql
	 * @param id
	 * @return
	 * @throws PersistWorkException
	 */
	protected Object getObjecteSentencia(String id) throws PersistWorkException {
		String paquet;
		if (id == null)
			paquet = ConfiguracioEntorn
					.getValorPropietat(ConfiguracioEntorn.PROPS_PACKAGE);
		else
			paquet = ConfiguracioEntorn.getValorPropietat(id,
					ConfiguracioEntorn.PROPS_PACKAGE);

		int posI = sentencia.indexOf(" ");
		String nomTaula = sentencia.substring(posI,
				sentencia.indexOf(" ", posI + 1));// .trim();

		String nomClasse = paquet + "." + nomTaula.trim();

		Class<?> clase;
		try {
			clase = Class.forName(nomClasse);
			obj = clase.newInstance();
			return obj;
		} catch (ClassNotFoundException e) {
			throw new PersistWorkException(PersistWorkException.NUC_3,
					PersistWorkException.NUC3_TXT);
		} catch (InstantiationException e) {
			throw new PersistWorkException(PersistWorkException.NUC_4,
					PersistWorkException.NUC4_TXT);
		} catch (IllegalAccessException e) {
			throw new PersistWorkException(PersistWorkException.NUC_5,
					PersistWorkException.NUC5_TXT);
		}
		
	}

}
