package com.smg.persistwork.utilscerca;

import java.util.ArrayList;
import java.util.List;

/**
 * Filtre: serveix per a realitzar consultes afegint un filtre.
 * @author Sergio Maeso
 * @version 1.0
 *
 */
public class Filtre {

	
	/**
	 * Restriccions que t el filtre
	 */
	private List<Restriccio> restriccions;
	
	/**
	 * Tipus de restriccions.
	 */
	private List<TipusFiltre> tipusRestriccions;
	
	/**
	 * Ordenaci que hi ha al filtre
	 */
	private List<String> ordenacio;
	
	/**
	 * Tipus d'ordenaci
	 */
	private List<TipusOrdre> tipusOrdre;

	/**
	 * 
	 */
	public Filtre() {
		super();
		restriccions = new ArrayList<Restriccio>();
		tipusRestriccions = new ArrayList<TipusFiltre>();
		ordenacio = new ArrayList<String>();
		tipusOrdre = new ArrayList<TipusOrdre>();
	}
	
	/**
	 * Afegir una restricci de tipus AND
	 * @param rest
	 */
	public void addOpcioAnd(Restriccio rest){
		
		restriccions.add(rest);
		tipusRestriccions.add(TipusFiltre.AND);
	}
	
	/**
	 * Afegir una restricci de tipus OR.
	 * @param rest
	 */
	public void addOpcioOr(Restriccio rest){
		
		restriccions.add(rest);
		tipusRestriccions.add(TipusFiltre.OR);
	}
	
	
	/**
	 * Ordenar per aquest camp de forma ascendent
	 * @param nomCamp
	 */
	public void addOrdenacioAsc(String nomCamp){
		
		ordenacio.add(nomCamp);
		tipusOrdre.add(TipusOrdre.ASCENDENT);
	}
	
	
	/**
	 * Ordenar per aquest camp de forma descendent
	 * @param nomCamp
	 */
	public void addOrdenacioDesc(String nomCamp){
		
		ordenacio.add(nomCamp);
		tipusOrdre.add(TipusOrdre.DESCENDENT);
	}
	
	/**
	 * Retorna la condici per a afegir a la sentncia.
	 * @return sentncia del filtre
	 */
	public String getSentencia(){
		
		StringBuilder st =new StringBuilder();
		
		if (restriccions.size()>0){
			st.append(" WHERE ");
			for(int i =0;i<restriccions.size();i++){
				if (i!=0){
					if (tipusRestriccions.get(i)==TipusFiltre.AND)
						st.append(" AND ");
					else
						st.append(" OR ");
				} 
				st.append(restriccions.get(i).getSentencia());
			}
		}
		
		if (ordenacio.size()>0){
			st.append(" ORDER BY ");
			for(int i =0;i<ordenacio.size();i++){
				if (i!=0){
					st.append(",");
				}
				st.append("@#").append(ordenacio.get(i)).append("@# ");
				if (tipusOrdre.get(i) == TipusOrdre.ASCENDENT)
					st.append("ASC");
				else
					st.append("DESC");
			}
		}
		return st.toString();
	}
}
