package com.smg.persistwork.nucli;

import static org.junit.Assert.*;


import junit.framework.Assert;

import org.junit.Test;

import com.smg.persistwork.aplicacio.model.AdministracionTest;
import com.smg.persistwork.aplicacio.model.EmpleadoTest;
import com.smg.persistwork.connexio.bd.ConnexioBDMySql;
import com.smg.persistwork.connexio.bd.ConnexioBDOracle;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.nucli.EsquemaAplicacio;

public class EsquemaAplicacioMySqlTest {

	@Test
	public void testEsquemaAplicacio() {
		try {
			EsquemaAplicacio esq = new EsquemaAplicacio();
			Assert.assertNull(esq.getId());
			Assert.assertNotNull(esq.getConnexio());
			Assert.assertTrue(esq.getConnexio() instanceof ConnexioBDOracle);
		
		} catch (PersistWorkException e) {
			
			fail(e.getMessage());
		}
	}

	@Test
	public void testEsquemaAplicacioString() {
		try {			
			EsquemaAplicacio esq2 = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
			Assert.assertNotNull(esq2.getId());
			Assert.assertEquals("mysqlBD",esq2.getId());
			Assert.assertNotNull(esq2.getConnexio());
			Assert.assertFalse(esq2.getConnexio() instanceof ConnexioBDOracle);
			Assert.assertTrue(esq2.getConnexio() instanceof ConnexioBDMySql);
		} catch (PersistWorkException e) {
			
			fail(e.getMessage());
		}
	}

	@Test
	public void testCreaEliminaTaula() {
		try {
			
			EsquemaAplicacio esq = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
		
			Assert.assertTrue(esq.getConnexio() instanceof ConnexioBDMySql);
		
			esq.creaTaula(new AdministracionTest());
			esq.creaTaula(new EmpleadoTest());
			esq.eliminaTaula(new EmpleadoTest());
			esq.eliminaTaula(new AdministracionTest());
		} catch (PersistWorkException e) {
			
			fail(e.getMessage());
		} 
	}
	
	@Test
	public void testNuevoCreaEliminaBD() {
		try {
			
			EsquemaAplicacio esq  = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
		
			esq.creaBaseDades();
			esq.eliminaBaseDades();

		} catch (PersistWorkException e) {
			
			fail(e.getMissatge());
		} 
	}
		
	
	@Test
	public void testgetNomEntitats() {
		try {
			EsquemaAplicacio esq  = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
			Object[] objs = esq.getNomEntitats();
			Assert.assertNotNull(objs);
			Assert.assertSame(2,objs.length);
			Assert.assertTrue(objs[1] instanceof EmpleadoTest);
			Assert.assertTrue(objs[0] instanceof AdministracionTest);
						
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			fail(e.getMissatge());
		}
	}
	

}
