package com.smg.persistwork.nucli;

import static org.junit.Assert.fail;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;

import com.smg.persistwork.aplicacio.model.AdministracionTest;
import com.smg.persistwork.aplicacio.model.EmpleadoTest;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.model.mapeig.MapeigSql;
import com.smg.persistwork.utilscerca.Filtre;
import com.smg.persistwork.utilscerca.Restriccio;

public class ManegadorEntitatMySqlTest {



	@Test
	public void testNuevoCreaBD() {
		try {
			
			EsquemaAplicacio esq  = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
		
			esq.creaBaseDades();
			//esq.eliminaBaseDades();

		} catch (PersistWorkException e) {
			
			fail(e.getMissatge());
		} 
	}
	
	@Test
	public void testInserta() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			AdministracionTest adm = new AdministracionTest(8,"Nuevas adminis");
			//NuevoAdministracion adm2 = new NuevoAdministracion(8,"Adm. nova");
			mng.inserta(adm);		
			//mng.inserta(adm2);		
			mng.inserta(adm);	
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	
	@Test
	public void testInicialitzaTransaccion() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest("Nom empleat",22,2222l,new Integer(45),new Long(77778),45.6,4444.78f,new Double(2222.3),new Float(3333.45), BigDecimal.TEN,new Date(),new GregorianCalendar(),new AdministracionTest(1,"test"));
			mng.inserta(emp);
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	@Test
	public void testFerCommit() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest("Nombre generics 2!",22,2222l,new Integer(45),new Long(77778),45.6,4444.78f,new Double(2222.3),new Float(3333.45), BigDecimal.TEN,new Date(),new GregorianCalendar(),new AdministracionTest(1,"test"));
			mng.inicialitzaTransaccion();
			mng.inserta(emp);
			mng.inserta(emp);
			mng.inserta(emp);
			mng.inserta(emp);
			mng.ferCommit();
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	@Test
	public void testFerRollBack() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest("Nombre rollback!",22,2222l,new Integer(45),new Long(77778),45.6,4444.78f,new Double(2222.3),new Float(3333.45), BigDecimal.TEN,new Date(),new GregorianCalendar(),new AdministracionTest(1,"test"));
			mng.inicialitzaTransaccion();
			mng.inserta(emp);
			mng.inserta(emp);
			mng.inserta(emp);
			mng.inserta(emp);
			mng.ferRollBack();
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	@Test
	public void testModifica() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest();
			AdministracionTest adm = new AdministracionTest(1,"jajaja");
			adm = mng.consultaPerClau(adm);
			adm.setDescripcio("Nova descripci genric");
			mng.modifica(adm);
			
			adm = mng.consultaPerClau(adm);
			
			emp.setId3(1);
			emp = mng.consultaPerClau(emp);
			emp.setNom("Nou empleat generic");
			mng.modifica(emp);

			EmpleadoTest emp2 = new EmpleadoTest();
			emp2.setId3(1);
			emp2 = mng.consultaPerClau(emp2);
			
			//Assert.assertEquals("Nova descripci", adm.getDescripcio());
			Assert.assertNotNull(emp);
			//Assert.assertEquals("Nou empleat 26", emp2.getNom());
		} catch (PersistWorkException e) {
				
			Assert.fail(e.getMissatge());
		}
	}

	@Test
	public void testEsborra() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest(null,22,2222l,new Integer(8),new Long(77778),45.6,4444.78f,new Double(2222.3),new Float(3333.45), BigDecimal.TEN,new Date(),new GregorianCalendar(),new AdministracionTest(2,"test"));
			
		
			emp.setId3(4);
	
			emp = mng.consultaPerClau(emp);
			Assert.assertNotNull(emp.getNom());
			mng.esborra(emp);
			emp = new EmpleadoTest();
			emp.setId3(4);
			emp = mng.consultaPerClau(emp);
			Assert.assertNull(emp);
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	@Test
	public void testConsultaPerClau() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			EmpleadoTest emp = new EmpleadoTest(null,22,2222l,new Integer(8),new Long(77778),45.6,4444.78f,new Double(2222.3),new Float(3333.45), BigDecimal.TEN,new Date(),new GregorianCalendar(),new AdministracionTest(2,"test"));
			AdministracionTest adm = new AdministracionTest(1,"jajaja");
			//emp.
			emp.setId3(1);

			emp = mng.consultaPerClau(emp);
			adm = mng.consultaPerClau(adm);
			//Assert.assertEquals("Nova descripci", adm.getDescripcio());
			Assert.assertNotNull(emp);
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}
	
	
	@Test
	public void testCerca() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			AdministracionTest adm = new AdministracionTest();
			adm.setDescripcio("Nuevas adminis");
			List<AdministracionTest> res = mng.cerca(adm);
			
			Assert.assertNotNull(res);
			Assert.assertNotSame(0, res.size());
					
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}
	
	@Test
	public void testCercaFiltre() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			AdministracionTest adm = new AdministracionTest();
		
			Restriccio rest = Restriccio.igual("descripcio", "Nuevas adminis");
			Restriccio rest2 = Restriccio.menorque("id", 3);
			Filtre filt = new Filtre();
			filt.addOpcioAnd(rest);
			filt.addOpcioAnd(rest2);
			List<AdministracionTest> res = mng.cerca(adm,filt);
			
			Assert.assertNotNull(res);
			Assert.assertNotSame(0, res.size());
					
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}

	
	@Test
	public void testCercaSqlNativa() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			AdministracionTest adm = new AdministracionTest();
		
			MapeigSql<AdministracionTest> mapeig = new MapeigSql<AdministracionTest>(adm,"mysqlBD");
			Map<String,String> map = new HashMap<String,String>();
			map.put("IDENTIFICADOR", "id");
			map.put("DESCRIPCION","descripcio");
			mapeig.setCamps(map);
			String sqlNativa = "SELECT ID AS IDENTIFICADOR, DESCRIPCIO AS DESCRIPCION FROM ADMINISTRACIONS WHERE ID<9";
			List<AdministracionTest> res = mng.consultaSqlNativa(sqlNativa, adm, mapeig);
					
		
			
			Assert.assertNotNull(res);
		
			Assert.assertNotSame(0, res.size());
		
					
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}
	
	@Test
	public void testCercaFwkql() {
		try {
			ManegadorEntitat mng = ManegadorEntitatFactory.createManegadorEntitat("mysqlBD","META-INF/fitxer-test.properties");
			
			String consulta = "select AdministracionTest on :id<? i :descripcio='Nuevas adminis'";
			List<Object> objs = new ArrayList<Object>(); 
			objs.add(3);
			//objs.add("Descripcion");
			
			List<AdministracionTest> res = mng.consultaFwkql(consulta,objs);
			
			String consulta2 = "select EmpleadoTest on (:nom semblant ? i :admin.id=?)";
			List<Object> objs2 = new ArrayList<Object>(); 
			objs2.add("Nou empleat%");
			objs2.add(1);
			
			List<EmpleadoTest> res2 = mng.consultaFwkql(consulta2,objs2);
				
			Assert.assertNotNull(res);
			Assert.assertNotNull(res2);
			Assert.assertNotSame(0, res.size());
			Assert.assertNotSame(0, res2.size());
					
		} catch (PersistWorkException e) {
			// TODO Auto-generated catch block
			Assert.fail(e.getMissatge());
		}
	}
	
	
	@Test
	public void testEliminaBD() {
		try {
			
			EsquemaAplicacio esq  = new EsquemaAplicacio("mysqlBD","META-INF/fitxer-test.properties");
		
			//esq.creaBaseDades();
			esq.eliminaBaseDades();

		} catch (PersistWorkException e) {
			
			fail(e.getMissatge());
		} 
	}
}
