package com.smg.persistwork.aplicacio.rrhh.accions;

import java.util.List;

import com.smg.persistwork.aplicacio.rrhh.model.Departament;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.nucli.ManegadorEntitat;
import com.smg.persistwork.nucli.ManegadorEntitatFactory;
import com.smg.persistwork.utilscerca.Filtre;
import com.smg.persistwork.utilscerca.Restriccio;

/**
 * @author Sergio Maeso
 *
 */
public class DepartamentAccio {

	/**
	 * 
	 */
	private ManegadorEntitat mng;

	/**
	 * @throws PersistWorkException
	 */
	public DepartamentAccio() throws PersistWorkException {
		super();
		mng = ManegadorEntitatFactory.createManegadorEntitat();
	}
	
	
	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public Departament insertaDepartament(String nom, String correu, String adreca, String numEmpleats) throws PersistWorkException{
		
		
		if ("".equals(nom) || nom==null){
			throw new PersistWorkException(400, "El nom s obligatori");
		}
		
		if ("".equals(correu) || correu==null){
			throw new PersistWorkException(400, "El correu s obligatori");
		}
		
		if ("".equals(adreca) || adreca==null){
			throw new PersistWorkException(400, "L'adreca s obligatori");
		}
		int numemp = 0;
		try{
			numemp = Integer.parseInt(numEmpleats);
		} catch (NumberFormatException num){
			throw new PersistWorkException(400, "El numero d'empleats ha de ser numric");
		}
			
	
		
		Departament dpt = new Departament();
		dpt.setNom(nom);
		dpt.setCorreuElectronic(correu);
		dpt.setAdreca(adreca);
		dpt.setNumEmpleats(numemp);
		
		mng.inserta(dpt);
		
		return dpt;
	}
	
	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public Departament guardaDepartament(String id, String nom, String correu, String adreca, String numEmpleats) throws PersistWorkException{
		
		Departament dpt = new Departament();
		
		if ("".equals(nom) || nom==null){
			throw new PersistWorkException(400, "El nom s obligatori");
		}
		
		if ("".equals(correu) || correu==null){
			throw new PersistWorkException(400, "El correu s obligatori");
		}
		
		if ("".equals(adreca) || adreca==null){
			throw new PersistWorkException(400, "L'adreca s obligatori");
		}
		int numemp = 0;
		try{
			numemp = Integer.parseInt(numEmpleats);
		} catch (NumberFormatException num){
			throw new PersistWorkException(400, "El numero d'empleats ha de ser numric");
		}
			
		int numdpt = 0;
		try{
			numdpt = Integer.parseInt(id);		
			dpt.setId(numdpt);
			
		
		} catch (NumberFormatException num){
			throw new PersistWorkException(400, "L'identificador del departament ha de ser numric");
		}
				
		dpt.setNom(nom);
		dpt.setCorreuElectronic(correu);
		dpt.setAdreca(adreca);
		dpt.setNumEmpleats(numemp);
		
		mng.modifica(dpt);
		
		return dpt;
	}
	
	/**
	 * @return
	 * @throws PersistWorkException 
	 */
	public Departament consultaDepartament(String id) throws PersistWorkException{
		
		int numdpt = 0;
		try{
			numdpt = Integer.parseInt(id);
			Departament dpt = new Departament();
			dpt.setId(numdpt);
			return mng.consultaPerClau(dpt);
		
		} catch (NumberFormatException num){
			throw new PersistWorkException(400, "L'identificador del departament ha de ser numric");
		}
		
	}
	
	/**
	 * @return
	 * @throws PersistWorkException 
	 */
	public Departament esborraDepartament(String id) throws PersistWorkException{
		
		int numdpt = 0;
		try{
			numdpt = Integer.parseInt(id);
			Departament dpt = new Departament();
			dpt.setId(numdpt);
			mng.esborra(dpt);
			return new Departament();
		
		} catch (NumberFormatException num){
			throw new PersistWorkException(400, "L'identificador del departament ha de ser numric");
		}
		
	}
	
	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<Departament> cercaDepartaments() throws PersistWorkException{
		
		return mng.cercaTots(new Departament());
	}
	
	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<Departament> cercaDepartamentsCamps(String nom, String numEntre, String numFi, String campOrd, String tipOrd) throws PersistWorkException{
		
		Filtre filt = new Filtre();
		if (nom!=null && !"".equals(nom))
			filt.addOpcioAnd(Restriccio.igual("nom", nom));
		
		if (numEntre!=null && !"".equals(numEntre)){
			int numemp = 0;
			try{
				numemp = Integer.parseInt(numEntre);
				filt.addOpcioAnd(Restriccio.igualmajorque("numEmpleats", numemp));
			} catch (NumberFormatException num){
				throw new PersistWorkException(400, "El numero d'empleats entre ha de ser numric");
			}
		}
		
		if (numFi!=null && !"".equals(numFi)){
			int numemp = 0;
			try{
				numemp = Integer.parseInt(numFi);
				filt.addOpcioAnd(Restriccio.igualmenorque("numEmpleats", numemp));
			} catch (NumberFormatException num){
				throw new PersistWorkException(400, "El numero d'empleats fins ha de ser numric");
			}
		}
		
		if (campOrd!=null && !"0".equals(campOrd)){
			if ("desc".equals(tipOrd)){
				filt.addOrdenacioDesc(campOrd);
			} else{
				filt.addOrdenacioAsc(campOrd);
			}
		}
			
			
		
		return mng.cerca(new Departament(), filt);
	}
	
}
