package com.smg.persistwork.aplicacio.rrhh.accions;

import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.List;

import com.smg.persistwork.aplicacio.rrhh.model.Empleat;
import com.smg.persistwork.aplicacio.rrhh.model.Projecte;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.nucli.ManegadorEntitat;
import com.smg.persistwork.nucli.ManegadorEntitatFactory;

/**
 * @author Sergio Maeso
 * 
 */
public class ProjecteAccio {

	/**
	 * 
	 */
	private ManegadorEntitat mng;

	/**
	 * @throws PersistWorkException
	 */
	public ProjecteAccio() throws PersistWorkException {
		super();
		mng = ManegadorEntitatFactory.createManegadorEntitat();
	}

	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public Projecte inserta(String identificador,String nom, String descripcio, String dataInici,
			String dataFi, String horesValorades, String capProjecte)
			throws PersistWorkException {

		
		if ("".equals(identificador) || identificador == null) {
			throw new PersistWorkException(400, "El identificador s obligatori");
		}
		if ("".equals(nom) || nom == null) {
			throw new PersistWorkException(400, "El nom s obligatori");
		}

		if ("".equals(descripcio) || descripcio == null) {
			throw new PersistWorkException(400, "La descripci s obligatoria");
		}

		int hores = 0;
		try {
			hores = Integer.parseInt(horesValorades);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"Les hores han de ser numriques");
		}

		SimpleDateFormat form1 = new SimpleDateFormat("dd/MM/yyyy");
		GregorianCalendar calI = new GregorianCalendar();
		GregorianCalendar calF = new GregorianCalendar();
		try {
			calI.setTime(form1.parse(dataInici));
			calF.setTime(form1.parse(dataFi));
		} catch (Exception e) {
			throw new PersistWorkException(400,
					"El format de les dates s : dd/MM/yyyy");
		}

		int idE = 0;
		try {
			idE = Integer.parseInt(capProjecte);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"El cap projecte de ser numric");
		}

		Projecte pjt = new Projecte();
		pjt.setId(identificador);
		pjt.setNom(nom);
		pjt.setDescripcio(descripcio);
		pjt.setDataFi(calF);
		pjt.setDataInici(calI);
		pjt.setEstat("ESBORRANY");
		pjt.setHoresImputades(0);
		pjt.setHoresValorades(hores);

		Empleat emp = new Empleat();
		emp.setId(idE);
		pjt.setCapProjecte(emp);

		mng.inserta(pjt);

		return pjt;
	}

	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public Projecte guarda(String id, String nom, String descripcio,
			String dataInici, String dataFi, String estat, String horesValorades,
			String horesImputades, String capProjecte) throws PersistWorkException {

		if ("".equals(nom) || nom == null) {
			throw new PersistWorkException(400, "El nom s obligatori");
		}

		if ("".equals(descripcio) || descripcio == null) {
			throw new PersistWorkException(400, "La descripci s obligatoria");
		}

		int horesI = 0;
		int horesV = 0;
		try {
			horesI = Integer.parseInt(horesImputades);
			horesV = Integer.parseInt(horesValorades);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"Les hores han de ser numriques");
		}

		SimpleDateFormat form1 = new SimpleDateFormat("dd/MM/yyyy");
		GregorianCalendar calI = new GregorianCalendar();
		GregorianCalendar calF = new GregorianCalendar();
		try {
			calI.setTime(form1.parse(dataInici));
			calF.setTime(form1.parse(dataFi));
		} catch (Exception e) {
			throw new PersistWorkException(400,
					"El format de les dates s : dd/MM/yyyy");
		}

		int idE = 0;
		try {
			idE = Integer.parseInt(capProjecte);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"El cap projecte de ser numric");
		}

		Projecte pjt = new Projecte();
		pjt.setNom(nom);
		pjt.setId(id);
		pjt.setDescripcio(descripcio);
		pjt.setDataFi(calF);
		pjt.setDataInici(calI);
		pjt.setEstat(estat);
		pjt.setHoresImputades(horesI);
		pjt.setHoresValorades(horesV);

		Empleat emp = new Empleat();
		emp.setId(idE);
		pjt.setCapProjecte(emp);

		mng.modifica(pjt);

		return pjt;
	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public Projecte consulta(String id) throws PersistWorkException {

		Projecte pjt = new Projecte();
		pjt.setId(id);
		return mng.consultaPerClau(pjt);

	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public Projecte esborra(String id) throws PersistWorkException {

		Projecte pjt = new Projecte();
		pjt.setId(id);
		mng.esborra(pjt);
		return new Projecte();

	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<Projecte> cerca() throws PersistWorkException {

		return mng.cercaTots(new Projecte());
	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<Projecte> cercaCamps(String nom, String estat, String capProjecte) throws PersistWorkException{
		
		if ((nom==null || "".equals(nom))
				&&  (estat==null || "".equals(estat)) &&  (capProjecte==null || "".equals(capProjecte)))
				return cerca();
		
		Projecte pjt = new Projecte();
		if (!"".equals(nom))
			pjt.setNom(nom);
		if (!"".equals(estat))
			pjt.setEstat(estat);
		if (!"".equals(capProjecte)){		
			int numemp = 0;
			Empleat emp = new Empleat();
			try{
				numemp = Integer.parseInt(capProjecte);
				
				emp.setId(numemp);				
			
			} catch (NumberFormatException num){
				throw new PersistWorkException(400,  "L'identificador de l'empleat ha de ser numric");
			}
			pjt.setCapProjecte(emp);			
		}
						
		return mng.cerca(pjt);
	}

}
