package com.smg.persistwork.aplicacio.rrhh.accions;

import java.util.List;

import com.smg.persistwork.aplicacio.rrhh.model.Empleat;
import com.smg.persistwork.aplicacio.rrhh.model.Projecte;
import com.smg.persistwork.aplicacio.rrhh.model.ProjecteEmpleat;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.nucli.ManegadorEntitat;
import com.smg.persistwork.nucli.ManegadorEntitatFactory;

/**
 * @author Sergio Maeso
 * 
 */
public class ProjecteEmpleatAccio {

	/**
	 * 
	 */
	private ManegadorEntitat mng;

	/**
	 * @throws PersistWorkException
	 */
	public ProjecteEmpleatAccio() throws PersistWorkException {
		super();
		mng = ManegadorEntitatFactory.createManegadorEntitat();
	}

	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public ProjecteEmpleat inserta(String empleat, String projecte, String participacio)
			throws PersistWorkException {


		ProjecteEmpleat pjemp = new ProjecteEmpleat();
		if ("".equals(empleat) || empleat == null) {
			throw new PersistWorkException(400, "L'empleat s obligatori");
		}
		if ("".equals(projecte) || projecte == null) {
			throw new PersistWorkException(400, "El projecte s obligatori");
		}
		
		double part = 0;
		try {
			if (participacio!=null && !"".equals(participacio))
				part = Double.parseDouble(participacio);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"La participaci ha de ser numrica");
		}



		int idE = 0;
		try {
			idE = Integer.parseInt(empleat);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"L'empleat ha de ser numric");
		}

		Projecte pjt = new Projecte();
		pjt.setId(projecte);
		
		Empleat emp = new Empleat();
		emp.setId(idE);
		
		pjemp.setParticipacio(part);
		pjemp.setEmpleat(emp);
		pjemp.setProjecte(pjt);
		

		mng.inserta(pjemp);

		return pjemp;
	}

	/**
	 * @param nom
	 * @param correu
	 * @param adreca
	 * @param numEmpleats
	 * @return
	 * @throws PersistWorkException
	 */
	public ProjecteEmpleat guarda(String id, String empleat, String projecte, String participacio) throws PersistWorkException {

		ProjecteEmpleat pjemp = new ProjecteEmpleat();
		if ("".equals(id) || id == null) {
			throw new PersistWorkException(400, "L'id s obligatori");
		}
		if ("".equals(empleat) || empleat == null) {
			throw new PersistWorkException(400, "L'empleat s obligatori");
		}
		if ("".equals(projecte) || projecte == null) {
			throw new PersistWorkException(400, "El projecte s obligatori");
		}
		
		double part = 0;
		try {
			if (participacio!=null && !"".equals(participacio))
				part = Double.parseDouble(participacio);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"La participaci ha de ser numrica");
		}

		int idR = 0;
		try {
			idR = Integer.parseInt(id);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"L'identificador ha de ser numric");
		}
		
		int idE = 0;
		try {
			idE = Integer.parseInt(empleat);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"L'empleat ha de ser numric");
		}

		Projecte pjt = new Projecte();
		pjt.setId(projecte);
		
		Empleat emp = new Empleat();
		emp.setId(idE);
		
		pjemp.setParticipacio(part);
		pjemp.setEmpleat(emp);
		pjemp.setProjecte(pjt);
		pjemp.setId(idR);
		

		mng.modifica(pjemp);

		return pjemp;
	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public ProjecteEmpleat consulta(String id) throws PersistWorkException {

		ProjecteEmpleat pjt = new ProjecteEmpleat();
		int idE = 0;
		try {
			idE = Integer.parseInt(id);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"L'id ha de ser numric");
		}

		pjt.setId(idE);
		return mng.consultaPerClau(pjt);

	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public ProjecteEmpleat esborra(String id) throws PersistWorkException {

		ProjecteEmpleat pjt = new ProjecteEmpleat();
		int idE = 0;
		try {
			idE = Integer.parseInt(id);
		} catch (NumberFormatException num) {
			throw new PersistWorkException(400,
					"L'id ha de ser numric");
		}

		pjt.setId(idE);
		mng.esborra(pjt);
		return new ProjecteEmpleat();

	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<ProjecteEmpleat> cerca() throws PersistWorkException {

		return mng.cercaTots(new ProjecteEmpleat());
	}

	/**
	 * @return
	 * @throws PersistWorkException
	 */
	public List<ProjecteEmpleat> cercaCamps(String empleat, String projecte) throws PersistWorkException{
		
		if ((empleat==null || "".equals(empleat))
				&&  (projecte==null || "".equals(projecte)))
				return cerca();
		
		ProjecteEmpleat pjtemp = new ProjecteEmpleat();

		if (!"".equals(empleat)){		
			int numemp = 0;
			Empleat emp = new Empleat();
			try{
				numemp = Integer.parseInt(empleat);
				
				emp.setId(numemp);				
			
			} catch (NumberFormatException num){
				throw new PersistWorkException(400,  "L'identificador de l'empleat ha de ser numric");
			}
			pjtemp.setEmpleat(emp);			
		}
		
		if (!"".equals(projecte)){		
			
			Projecte pjte = new Projecte();
			
			pjte.setId(projecte);		
			pjtemp.setProjecte(pjte);
		}
						
		return mng.cerca(pjtemp);
	}

}
